
/*=====================================================================*
 * Program HABSUM.SAS                                                  *
 * Version 1.0:  01 March 2001                                         *
 *                                                                     *
 * Program HABSUM.SAS.  This program serves as an example to analyze   *
 * habitat use data from HABUSE ARC/INFO AML.  It illustrates analysis *
 * of wild turkey data collected as part of the Wisconsin Department   *
 * of Natural Resources' Bureau of Research Project RS-234; Wild       *
 * Turkey Populations and Management.  The infile 'xmplout.txt'        *
 * contains the results of using Samuel and Kenow's (1992, J. Wildl.   *
 * Manage. 56:725-734) method to estimate and accommodate habitat      *
 * misclassification error; then extracting and summarizing the        *
 * habitat type at each random point through the ARC/INFO commands     *
 * 'identity' and 'frequency', respectively. Habitat selection is      *
 * tested for using the method presented by Neu et al. (1975, J.       *
 * Wildl. Manage. 38:541-545) and Byers et al. (1984, J. Wildl.        *
 * Manage. 48:1050-1053).                                              *
 *                                                                     *
 *                                                                     *
 *                                                                     *
 * For information pertaining to the use of this program contact:      *
 *                                                                     *
 *   Bob Wright                                                        *
 *   Midwest Bio-Services                                              *
 *   N6685 CTH M                                                       *
 *   Holmen, WI 54636                                                  *
 *   TEL: 608-526-2511                                                 *
 *   EMAIL: midwbiosrv@aol.com                                         *
 *                                                                     *
 *         or                                                          *
 *                                                                     *
 *   Kevin P. Kenow                                                    *
 *   Upper Midwest Environmental Sciences Center                       *
 *   2630 Fanta Reed Road                                              *
 *   La Crosse, WI  54603                                              *
 *   TEL:  608-781-6278                                                *
 *   FAX:  608-783-6066                                                *
 *   EMAIL:  kevin_kenow@usgs.gov                                      *
 *                                                                     *
 *=========================== HISTORY =================================*
 *                                                                     *
 *                                                                     *
 *  Author      Date       Event                                       *
 *  ------      ----       -----                                       *
 *  Wright      12/15/97   Original coding                             *
 *  KPK         01/29/98   Modified for use as sample program with     *
 *                         with SUBSAMPL/HABUSE software               *
 *  KPK         11/27/00   Modified 27 November 2000 with corrected    *
 *                         variance and 95% CI intervals               *
 *  KPK         03/01/01   Prepared for Center website distribution    *
 *                                                                     *
 *========================= DISCLAIMER  ===============================*
 *                                                                     *
 *  Although this program has been used by the U.S. Geological Survey  *
 *  (USGS), no warranty, expressed or implied, is made by the USGS as  *
 *  to the accuracy and functioning of the program and related program *
 *  material, nor shall the fact of distribution constitute any such   *
 *  warranty, and no responsibility is assumed by the USGS in          *
 *  connection therewith.                                              *
 *                                                                     *
 *=====================================================================*/

/***********************************************************************
 * Users should input the number of habitat types (k) and the alpha    *
 * level for the Bonferroni simultaneous confidence interval (Byers et *
 * al. 1984, J. Wildl. Manage. 48:1050-1053)                           *
 ***********************************************************************/


/****REASSIGNING COUNT TO NEW HABITAT VARIABLE, PREPPING FOR MERGE****/

DATA habuse1;
   infile 'd:\turkey\example_16nov00\crop_subset.txt' delimiter=',';
   input n1 count birdid date time habitat $;
   birdid = 18003;
   k = 4;   /* number of habitat types in analysis */
   alpha = 0.05;   /* alpha level for Bonferroni confidence interval */
   Z = probit(1.0 - alpha / (2 *k));  /* Z value for Bonferroni CI */

     if habitat="'C'" then c=count;          /*Crop field*/
     if habitat="'F'" then f=count;          /*Forest*/
     if habitat="'P'" then p=count;          /*Pasture*/
     drop habitat count;
run;

PROC sort;
  by birdid date time;
run;

/****MERGING HABITAT TYPE COUNTS****/


DATA crop; set habuse1;
  if c>0 then do;
    keep birdid date time c z;
    output;
  end;
run;

DATA forest; set habuse1;
  if f>0 then do;
    keep birdid date time f z;
    output;
  end;
run;

DATA pasture; set habuse1;
  if p>0 then do;
    keep birdid date time p z;
    output;
  end;
run;

DATA habitmrg;
  merge crop forest pasture;
  by birdid date time;
run;

DATA habuse2; set habitmrg;
  if c<0 then c=0;
  if f<0 then f=0;
  if p<0 then p=0;
  total=c+f+p;
run;

PROC summary DATA=habuse2;
  var c;
  output out=broodcnt n=numobs;
run;

DATA habuse2;
  if _n_=1 then set broodcnt;
  set habuse2;
run;

DATA brood1; set habuse2;
  misc=(c/total)*(1-c/total)/(total);
  misf=(f/total)*(1-f/total)/(total);
  misp=(p/total)*(1-p/total)/(total);
run;

PROC PRINT; RUN;

PROC summary DATA=brood1;
  id numobs z;
  var c f p total misc misf misp;
  output out=brood2 sum=sumbc sumbf sumbp btotal
    bmisc bmisf bmisp;
run;

DATA brood3; set brood2;
  bmisc=bmisc/((numobs**2)*(btotal/numobs));
  propbc=sumbc/btotal;
  vsumbc=(propbc)*(1-propbc)/numobs;
  varbc=vsumbc+bmisc;
  cibc=Z*sqrt(varbc);
  lwrbc=propbc-cibc;
  uprbc=propbc+cibc;

  bmisf=bmisf/((numobs**2)*(btotal/numobs));
  propbf=sumbf/btotal;
  vsumbf=(propbf)*(1-propbf)/numobs;
  varbf=vsumbf+bmisf;
  cibf=Z*sqrt(varbf);
  lwrbf=propbf-cibf;
  uprbf=propbf+cibf;

  bmisp=bmisp/((numobs**2)*(btotal/numobs));
  propbp=sumbp/btotal;
  vsumbp=(propbp)*(1-propbp)/numobs;
  varbp=vsumbp+bmisp;
  cibp=Z*sqrt(varbp);
  lwrbp=propbp-cibp;
  uprbp=propbp+cibp;
run;

PROC PRINT; RUN;

                   /***PRINTING RESULTS***/


/***HABITAT USE ESTIMATES***/

proc print data=habuse1;
title 'HABITAT USE OF WILD TURKEY HEN #150.800';
run;

PROC print DATA=brood3;
  var propbc varbc lwrbc uprbc propbf varbf lwrbf uprbf propbp varbp lwrbp uprbp numobs;
run;
