
/*=====================================================================*
 * Program SUBSMPL.SAS                                                 *
 * Version 1.0:  01 March 2001                                         *
 *                                                                     *
 * SUBSAMPL was developed to conduct habitat use analyses of           *
 * wild turkey and duckling location data using SAS for Windows (6.12) *
 * (Copyright (c) 1989-1996 by SAS Institute Inc., Cary, NC, USA).     *
 * Input data include animal identification, date and time azimuth to  *
 * animal is recorded, location of receiver, and azimuth to animal.    *
 * These data are grouped sequentially in an ASCII input file by sets  *
 * of observations that determine a given location.  SUBSAMPL          *
 * calculates a confidence interval for the location solution          *
 * (White and Garrott 1990) of each set of bearings by maximum         *
 * likelihood procedures (Lenth 1981) using the standard deviation of  *
 * the directional bearings (Lee et al. 1985, White and Garrott 1990). *
 * A matrix that behaves like the square root of the                   *
 * variance-covariance matrix is produced and used to generate a       *
 * subsample of points from a bivariate normal distribution.  The      *
 * coordinates and attributes (animal identification, etc.) for these  *
 * subsample points are formatted into separate ARC/INFO input files.  *
 *                                                                     *
 *                                                                     *
 * For information pertaining to the use of this program contact:      *
 *                                                                     *
 *   Kevin P. Kenow                                                    *
 *   Upper Midwest Environmental Sciences Center                       *
 *   2630 Fanta Reed Road                                              *
 *   La Crosse, WI  54603                                              *
 *   TEL:  608-781-6278                                                *
 *   FAX:  608-783-6066                                                *
 *   EMAIL:  kevin_kenow@usgs.gov                                      *
 *                                                                     *
 *=========================== HISTORY =================================*
 *                                                                     *
 *                                                                     *
 *  Author      Date       Event                                       *
 *  ------      ----       -----                                       *
 *  KPK         01/31/00   Original coding                             *
 *  KPK         03/01/01   Prepared for Center website distribution    *
 *                                                                     *
 *========================= DISCLAIMER  ===============================*
 *                                                                     *
 *  Although this program has been used by the U.S. Geological Survey  *
 *  (USGS), no warranty, expressed or implied, is made by the USGS as  *
 *  to the accuracy and functioning of the program and related program *
 *  material, nor shall the fact of distribution constitute any such   *
 *  warranty, and no responsibility is assumed by the USGS in          *
 *  connection therewith.                                              *
 *                                                                     *
 *=====================================================================*/

/*=====================================================================*
 * Edit the following section to make choices appropriate for your     *
 * data.                                                               *
 *   sdangle is the standard deviation of the azimuth appropriate for  *
 *     your study (see Lee et al.  1985.  J. Wildl. Manage.            *
 *     49:658-663).                                                    *
 *   Two types of data entry are possible.                             *
 *     Data entry type 1:  Input telemetry data from one file (e.g.    *
 *                         output from program XYLOG)                  *
 *     Data entry type 2:  Data entry using separate telemetry data    *
 *                         and receiver location files (e.g., output   *
 *                         from program Locate)                        *
 *=====================================================================*/

%let sdangle = 5.5;  * change 5.5 as appropriate;

%let etype = 1;      * choose type 1 or 2;

%let dsn1 = %str(c:\subsampl\test1.dat);* regarless of data entry type-;
                                          * indicate telemetry data file ;
                                          * e.g., test1.dat for type 1   ;
                                          *       test2.dat for type 2   ;

%let dsn2 = %str(c:\subsampl\rcvr.dat);   * if data entry type 2 is selected-:
                                          * indicate receiver location file  ;

%let dsn3 = %str(c:\subsampl\subsampl.crd);  * adjust output files ;
%let dsn4 = %str(c:\subsampl\subsampl.att);  * if necessary        ;


%macro entry;
  %if &etype = 1 %then
    %do;

/*=====================================================================*
 * Data entry type 1:  Input telemetry data from one file (e.g.        *
 *                     output from program XYLOG)                      *
 *=====================================================================*/

data telemdat;
 INFILE "&dsn1" missover lrecl=80;
 INPUT id
        date
        time
        easting
        northing
        azimuth;
  idtime = (id*1000) + date + (time*0.0001);
  if id='.' then delete;
  run;

    %end;
  %else
    %do;

/*=====================================================================*
 *  Data entry type 2:  Data entry using separate telemetry data and   *
 *                      receiver location files (e.g., output from     *
 *                      program Locate)                                *
 *=====================================================================*/

DATA reclocs;
  INFILE "&dsn2" missover lrecl=80;
  input label $ 1-4 easting 11-17 northing 19-26;
  easting = easting*1000;
  northing = northing*1000;
  run;

proc sort data=reclocs;
  by label;
  run;

data telemraw;
 INFILE "&dsn1" missover lrecl=80;
  input id date time label $ azimuth;
  run;

proc sort data=telemraw;
  by label;
  run;

data telemdat;
  merge telemraw reclocs;
  by label;
  if id = . then delete;
  idtime = (id*1000) + date + (time*0.0001);
  run;

    %end;
%mend entry;

%entry
run;

/*=====================================================================*
 *  Process data                                                       *
 *=====================================================================*/

proc sort data=telemdat;
  by date time id;
  run;

data subsampl;
array tel{3,6} rx1-rx6 ry1-ry6 az1-az6;
  set telemdat;
  by idtime notsorted;
  retain rx1-rx6 ry1-ry6 az1-az6 j count;
  retain pi 3.14159;
  retain nsub 100;
  array theta{6} th1-th6;
  array azmsub{6} azsub1-azsub6;
  array thetatru{6} tru1-tru6;
  array azmtru{6} aztru1-aztru6;
  array a{2,2} a11 a12 a21 a22;
  array b{2} b1 b2;
  array distx{6} dxx1-dxx6;
  array disty{6} dyy1-dyy6;

  if first.idtime then j=0;
  j=j+1;
  if j < 7 then tel{1,j} = easting;
  if j < 7 then tel{2,j} = northing;
  if j < 7 then tel{3,j} = azimuth;
  if last.idtime then
    do;
  count = j;

/*=====================================================================*
 * Calculate the estimated location solution for the                   *
 *  observed set of azimuths - follows lenth 2.6 formula               *
 *=====================================================================*/

      do l = 1 to count;
        theta{l} = ((90-tel{3,l})*pi/180);
      end;

/*================================*
 * Calculate mean x,y coordinates *
 * for set of receiver locations  *
 ================================*/

          x1 = 0;
          y1 = 0;
        do i = 1 to count;
          x1 = x1 + tel{1,i};
          y1 = y1 + tel{2,i};
          end;
         x1 = x1/count;
         y1 = y1/count;

      do iter = 1 to 20;
        do k1 = 1 to 2;
          b{k1} = 0.0;
          do k2 = 1 to 2;
            a{k1,k2} = 0.0;
          end;
        end;
        v2=0;
        do j = 1 to count;
          dx = x1 - tel{1,j};
          dy = y1 - tel{2,j};
          di = sqrt(dx*dx + dy*dy);
          si = sin(theta{j});
          ci = cos(theta{j});
          if iter eq 1 then do;
            ss = si;
            cs = ci;
          end;
            else do;
              ss = dy / (di**3);
              cs = dx / (di**3);
            end;
          z = si * tel{1,j} - ci * tel{2,j};
          a{1,1} = a{1,1} + si*ss;
          a{1,2} = a{1,2} - ci*ss;
          a{2,1} = a{2,1} - si*cs;
          a{2,2} = a{2,2} + ci*cs;
          v2 = v2 + ci*ss + si*cs;
          b{1} = b{1} + ss*z;
          b{2} = b{2} - cs*z;
        end;
        v2 = -v2/2;
      ysoln = (a{1,1}*b{2} - a{2,1}*b{1}) / (a{1,1}*a{2,2} - a{2,1}*a{1,2});
      xsoln = (b{1} - a{1,2}*ysoln) / a{1,1};
      if ((abs(xsoln-x1)/xsoln<0.000001) and (abs(ysoln-y1)/ysoln<0.000001))
        then
         goto lenth26a;
      x1=xsoln;
      y1=ysoln;
      end;
/*================================*
 * Deal with azimuth sets that do *
 * not converge                   *
 *================================*/
      conv = 0;
      output;
      goto noconv;
lenth26a:

/*================================*
 * Check negative determinant for *
 * non convergance                *
 *================================*/

  dt = (a{1,1}*a{2,2})-(a{2,1}*a{1,2});
  if dt le 0.0 then do;
    conv = 0;
    output;
    goto noconv;
    end;
  conv = 1;

/*=====================================================================*
 * Calculate distance and true theta from each receiver location to    *
 * transmitter location solution and area of 95% confidence ellipse    *
 *=====================================================================*/


  do i = 1 to count;
    distx{i} = xsoln - tel{1,i};
    disty{i} = ysoln - tel{2,i};
    thetatru{i} = atan(disty{i}/distx{i});
    if distx{i} < 0 & disty{i} < 0 then thetatru{i} = thetatru{i}-pi;
    else if distx{i}<0 & disty{i}>=0 then thetatru{i} = thetatru{i}+pi;
    if thetatru{i} < 0 then thetatru{i} = thetatru{i}+2*pi;
    azmtru{i} = 90-(thetatru{i}*180/pi);
    if azmtru{i} < 0 then azmtru{i} = azmtru{i} + 360;
    if azmtru{i} > 360 then azmtru{i} = azmtru{i} - 360;
    end;
  AA = exp((&sdangle*pi/180)**2*(-0.5));
  kappa = 1/(2*(1-AA)+(1-AA)**2*(0.48794-0.82905*AA-1.3915*AA**2)/AA);
  discr = a{1,1}*a{2,2}-a{1,2}*a{2,1};
  dets = (1./((a{1,1}*a{2,2}-(a{1,2}*a{2,1}))*kappa**2));
  dets = sqrt(abs(dets));
  ell95 = pi*cinv(0.95,2)*dets;
  ell99 = pi*cinv(0.99,2)*dets;
  elldif = ell99 - ell95;

/*=====================================================================*
 * Develop variance-covariance matrix and sqrt of var-covar matrix     *
 * to generate bivariate normal distribution                           *
 *=====================================================================*/

  invkappa = 1/kappa;
  sex= sqrt(abs(invkappa*a{2,2}/discr));
  sey= sqrt(abs(invkappa*a{1,1}/discr));
  corr = -(a{1,2}+a{2,1})/2/sqrt(a{1,1}*a{2,2});
  if corr>1 then corr=0.999;

  v11=sex**2;
  v12=corr*sex*sey;
  v21=v12;
  v22=sey**2;

  d1=v11;
  sex=sqrt(d1);
  l21=v21/d1;
  d2=v22-((l21**2)*d1);
  sey=sqrt(d2);
  vs11=sex;
  vs12=sex*l21;
  vs22=sey;
  vs21=0;

/*=====================================================================*
 * Construct ellipse, for diagnostic purposes                          *
 *=====================================================================*/

  varx = a{2,2}/discr/kappa;
  vary = a{1,1}/discr/kappa;
  covxy = -v2/discr/kappa;
  qb = -varx - vary;
  qc = varx*vary - covxy*covxy;
  if qb*qb > 4*qc then
    qd = sqrt(qb*qb - 4*qc);
    else qd=0;
  lambda1 = (-qb + qd) / 2;
  if lambda1 le 0.0 then do;
    conv = 0;
    output;
    goto noconv;
    end;
  lambda2 = (-qb - qd) /2;
  thetta = atan((lambda1 - varx) / covxy);
  sthetta = sin(thetta);
  cthetta = cos(thetta);

  semima95 = sqrt(cinv(0.95,2)*lambda1);
  semimi95 = sqrt(cinv(0.95,2)*lambda2);
  semima99 = sqrt(cinv(0.99,2)*lambda1);
  semimi99 = sqrt(cinv(0.99,2)*lambda2);

  step = 2*pi/40;
  do k = 0 to 40;
    alfa = step*k;
    tx5 = semima95 * cos(alfa);
    ty5 = semimi95 * sin(alfa);
    vx5 = (tx5*cthetta - ty5*sthetta) + xsoln;
    vy5 = (tx5*sthetta + ty5*cthetta) + ysoln;
    tx9 = semima99 * cos(alfa);
    ty9 = semimi99 * sin(alfa);
    vx9 = (tx9*cthetta - ty9*sthetta) + xsoln;
    vy9 = (tx9*sthetta + ty9*cthetta) + ysoln;
    output;
    end;


/*=====================================================================*
 *  Generate subsample of points from the bivariate                    *
 *  normal distribution generated above                                *
 *=====================================================================*/

    do k = 1 to nsub;
      bnsx1 = rannor(0);
      bnsx2 = rannor(0);
      bnsx = (bnsx1*vs11 + bnsx2*vs21) + xsoln;
      bnsy = (bnsx1*vs12 + bnsx2*vs22) + ysoln;
      disbnsx = xsoln-bnsx;
      disbnsy = ysoln-bnsy;
      disbns = sqrt(disbnsx*disbnsx + disbnsy*disbnsy);
      thetabns = atan(disbnsy/disbnsx);
      if disbnsx < 0 & disbnsy < 0 then thetabns = thetabns-pi;
      else if disbnsx<0 & disbnsy>=0 then thetabns = thetabns+pi;
      if thetabns < 0 then thetabns = thetabns+2*pi;
      output;
      end;  /* nsub */

/* Zero the array tel{k,i} */

      do i=1 to j-1;
        do k=1 to 3;
           tel{k,i} = .;
          end;
        end;
      j=1;
      end; /*if last.id*/
    noconv:
    keep id date time idtime k conv bnsx bnsy xsoln ysoln ell95 ell99
     vx5 vy5 vx9 vy9 rx1 rx2 rx3 rx4 rx5 rx6 ry1 ry2 ry3 ry4 ry5 ry6
     az1 az2 az3 az4 az5 az6;
    run;

/*=====================================================================*
 *  Format data for arcinfo                                            *
 *=====================================================================*/

data arcinfo;
  retain idntfier 0;
  set subsampl;
  if bnsx > 1;
  idntfier = idntfier+1;
  keep id date time k bnsx bnsy idntfier;
  format bnsx 6.0;
  format bnsy 7.0;
  file "&dsn3";
  put idntfier +(-1) ',' bnsx +(-1) ',' bnsy;
  file "&dsn4";
  put idntfier +(-1) ',' id +(-1) ',' date +(-1) ',' time +(-1);
  run;

data _null_;
  file "&dsn3" mod;
  put "end";
  run;

/*=====================================================================*
 *  Diagnostic information                                             *
 *=====================================================================*/

proc sort data=subsampl;
  by idtime;
  run;

data compare;
  set subsampl;
  by idtime;
  if last.idtime;
  if conv = 1;
  run;

/*================================*
 *  Print List of Solution Sets   *
 *================================*/

proc print data=compare;
  title 'Location Solutions and Subsamples Determined for the Following Azimuth Sets:';
  var id date time idtime conv k bnsx bnsy xsoln ysoln ell95 ell99;
  run;

data noconv;
  set subsampl;
  if conv =0;
  run;

/*================================*
 * Print List of Azimuth Sets for *
 * which there was no convergence *
 *================================*/

proc print data=noconv;
  title 'No Convergence for the Following Azimuth Sets:';
  var id date time conv rx1 rx2 rx3 rx4 rx5 rx6
      ry1 ry2 ry3 ry4 ry5 ry6 az1 az2 az3 az4 az5 az6;
  run;

/*=====================================================================*
 *  Display the relationship of receiver locations, location solution, *
 *  95% and 99% error ellipses, and subsample locations to one another *
 *  for selected examples.                                             *
 *=====================================================================*/

data s890717;
  set subsampl;
  if date = 890717;
  run;

goptions device=win display rotate=landscape fby=swiss hby=1.25;

title1 c=blue f=swiss 'Date=890717';
    symbol1 c=black mode=include v=star;
    symbol2 c=black mode=include v=plus h=1;
    symbol3 c=black mode=include v=plus h=1;
    symbol4 c=black mode=include v=plus h=1;
    symbol5 c=cyan mode=include i=join v=none;
    symbol6 c=blue mode=include i=join v=none;
    symbol7 c=red mode=include vaLUE=dot i=none h=.2;
    symbol8 c=green mode=include vaLUE=dot i=none h=.2;



PROC GPLOT DATA=s890717 uniform;
  axis1 label=(c=blue h=1.5 f=swiss "Easting")
        value=(c=blue h=1.5 f=swiss);
  axis2 label=(c=blue h=1.5 f=swiss j=c "Northing")
        value=(c=blue h=1.5 f=swiss);
  plot1 ysoln*xsoln ry1*rx1 ry2*rx2 ry3*rx3 vy5*vx5 vy9*vx9 bnsy*bnsx / overlay vaxis=axis2 haxis=axis1;
  run;

data s890719;
  set subsampl;
  if date = 890719;
  run;

goptions device=win display rotate=landscape fby=swiss hby=1.25;

title1 c=blue f=swiss 'Date=890719';
    symbol1 c=black mode=include v=star;
    symbol2 c=black mode=include v=plus h=1;
    symbol3 c=black mode=include v=plus h=1;
    symbol4 c=black mode=include v=plus h=1;
    symbol5 c=cyan mode=include i=join v=none;
    symbol6 c=blue mode=include i=join v=none;
    symbol7 c=red mode=include vaLUE=dot i=none h=.2;
    symbol8 c=green mode=include vaLUE=dot i=none h=.2;

PROC GPLOT DATA=s890719 uniform;
  axis1 label=(c=blue h=1.5 f=swiss "Easting")
        value=(c=blue h=1.5 f=swiss);
  axis2 label=(c=blue h=1.5 f=swiss j=c "Northing")
        value=(c=blue h=1.5 f=swiss);
  plot1 ysoln*xsoln ry1*rx1 ry2*rx2 ry3*rx3 vy5*vx5 vy9*vx9 bnsy*bnsx / overlay vaxis=axis2 haxis=axis1;
  run;

data s890822;
  set subsampl;
  if date = 890822;
  run;

goptions device=win display rotate=landscape fby=swiss hby=1.25;

title1 c=blue f=swiss 'Date=890822';
    symbol1 c=black mode=include v=star;
    symbol2 c=black mode=include v=plus h=1;
    symbol3 c=black mode=include v=plus h=1;
    symbol4 c=black mode=include v=plus h=1;
    symbol5 c=cyan mode=include i=join v=none;
    symbol6 c=blue mode=include i=join v=none;
    symbol7 c=red mode=include vaLUE=dot i=none h=.2;
    symbol8 c=green mode=include vaLUE=dot i=none h=.2;

PROC GPLOT DATA=s890822 uniform;
  axis1 label=(c=blue h=1.5 f=swiss "Easting")
        value=(c=blue h=1.5 f=swiss);
  axis2 label=(c=blue h=1.5 f=swiss j=c "Northing")
        value=(c=blue h=1.5 f=swiss);
  plot1 ysoln*xsoln ry1*rx1 ry2*rx2 ry3*rx3 vy5*vx5 vy9*vx9 bnsy*bnsx / overlay vaxis=axis2 haxis=axis1;
  run;
